;---------------------------------------------------------------------------
; BIKE GEARBOX SHIFTER CONTROLLER & 7Segment LED Display
;  Copyright 2000
;-----------------------------------------------------------------------------
; AUTHOR:     Marc Bell
; EMAIL:      mrb@cadence.com
; DATE:       Feb 2000
; DESCRIPTION:
;   Monitors 2 steering-wheel mounted switches and a neutral input.
;   Controls output solenoid driver FETs to move gearlever.
;   Current gear is displayed on a 7SEG LED display via a ShiftREG.
;   Optional outputs to Ignitor module (for full throttle shifts)
;   and clutch solenoid to allow full-auto.
;
; COPYRIGHT:
; You are free to use this software and the accompanying schematic for personal
; use only.  If you wish to use this for commercial gains you MUST contact the author
; for permission.  
;
;-----------------------------------------------------------------------------
;  VER    FILENAME   	NOTES
;  1.0    gearcont.asm	Original - No IGN Cut/Clutch drive
;-----------------------------------------------------------------------------
; HARDWARE SETUP
; -----------------------------------------------------------------------------
; PORT A: 0   Up-Solenoid            O/P    Pin17
;         1   Down-Solenoid          O/P 	Pin18
;         2   Neutral Switch         I/P    Pin1
;         3   Ignition Cutout        O/P    Pin2
;         4   Clutch Solenoid        O/P    Pin3    
;       TRIS_A => 0xb0001_1100
;  
; PORT B: 0   7SEG LED SERIAL DATA   O/P    Pin6
;         1   7SEG LED SERIAL CLOCK  O/P    Pin7
;         2   7SEG LED CLR_N         O/P    Pin8
;         3   spare
;         4   spare
;         5   spare
;         6   DOWN SWITCH           I/P    Pin12
;         7   UP SWITCH             I/P    Pin13    
;         TRIS_B => 0xb1100_0000
;
; Note on GearBox:
;  1st Gear is Onepush FORWARD from NEUTRAL
;  Subsequent upshifts are done by BACKWARD presses
;  Subsequent downshifts are done by more FORWARD presses
; 
;  Fosc = 3.6864MHz
;  Processor = 16F84
;
;    --A--     Common
;   |     |    Anode
;   F     B    Display
;   |     |    Used
;    --G--
;   |     |    Example:    Number 5 is ACDFG  Q(7:0)=X001_0010
;   E     C                Number 1 is BC     Q(7:0)=X111_1001
;   |     |
;   --D---
;
; Q0 is A
; Q1 is B
; Q2 is C
; Q3 is D
; Q4 is E
; Q5 is F
; Q6 is G
; Q7 is Not used
; ---------------------------------------------------------------------

	LIST P=16F84, F=INHX8M
	__CONFIG _CP_OFF & _WDT_OFF & _XT_OSC
	#include <p16f84.inc>
	#include <macros.inc>

;*****************************************************************************
; Equates, I/O, vars
;*****************************************************************************

#DEFINE		DIG_0 b'10111111'
#DEFINE		DIG_1 b'11111001'
#DEFINE		DIG_2 b'10100100'
#DEFINE		DIG_3 b'10110000'
#DEFINE		DIG_4 b'10011001'
#DEFINE		DIG_5 b'10010010'
#DEFINE		DIG_6 b'10000010'
#DEFINE		DIG_E b'10000110'

; PORTA bits
#DEFINE SOL_UP		PORTA, 0
#DEFINE SOL_DOWN	PORTA, 1 
#DEFINE NEU_SW		PORTA, 2

; PORTB bits
#DEFINE SER_DATA 	PORTB, 0
#DEFINE SER_CLK  	PORTB, 1
#DEFINE SER_CLRN	PORTB, 2
#DEFINE SW_DOWN		PORTB, 6
#DEFINE SW_UP		PORTB, 7

; Registers
GEAR		EQU	0x0C
DELAY		EQU	0x0D
GEAR_OUT	EQU	0x0E
i_COUNT		EQU	0x0F
X_DELAY		EQU	0x10
TMPREG		EQU	0x11
T_GEAR		EQU	0x12

;=============================================================================
; MACROS
;=============================================================================
DELAY_125MS	MACRO
		MOVLW	0xFF		; 125msec
		CALL	X_DELAY500
		ENDM

DELAY_250MS	MACRO
		DELAY_125MS		
		DELAY_125MS
		ENDM

CLOCKSTROBE 	MACRO
		NOP					; setup
  		BSF	SER_CLK
		NOP
		NOP					; hold
  		BCF	SER_CLK
		ENDM

;*****************************************************************************
; Program start
;*****************************************************************************
	ORG	0x00		; RESET vector location
RESET	GOTO	START


;*****************************************************************************
; ISR  (Should never occur)
	ORG	0x04		; Interrupt vector location
ISR	RETFIE                  ; back to calling routine


;*****************************************************************************
; S T A R T
;*****************************************************************************
START					; POWER_ON Reset (Beginning of program)
	CLRF	STATUS			; Do initialization, Select bank 0
	CLRF	INTCON			; Clear int-flags, Disable interrupts
	CLRF	PCLATH			; Keep in lower 2KByte
	CLRF	PORTA			
	CLRF	PORTB

	BANK1
	MOVLW	b'00011100'		; TRIS:  1=I/P   0=O/P
	MOVWF	TRISA
	MOVLW	b'11000000'
	MOVWF	TRISB
    MOVLW  	0x00		  	; DISABLE all interrupts
    MOVWF  	INTCON
	BANK0

	BSF		SOL_DOWN		; Turn the Solenoid Drivers OFF
	BSF		SOL_UP
	CLRF	GEAR
	CLRF	T_GEAR
	BCF		SER_CLRN		; Reset shift-register
	NOP
	NOP
	BSF		SER_CLRN
	CLRF	TMPREG
	
;*****************************************************************************
MAIN		
	CALL WIGGLER

SWTEST
	BTFSC	SW_DOWN 	; normally low 
	GOTO	DOWNGEAR	; down is pressed
	BTFSC	SW_UP 		; normally low
	GOTO 	UPGEAR		; up is pressed
	GOTO	SWTEST		; nowt pressed so loop

DOWNGEAR
	; 1st check to see if we are in Neutral
	MOVFW	GEAR
    MOVWF 	T_GEAR		; take a copy
    SUBLW 	D'0'		; take away 0 and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_1ST		; flag not set so check next gear
	GOTO	SWTEST		; flag set so GEAR=0 already 
CHK_1ST
	; Not in Neutral so find out if we're in 1st.	
	MOVFW	GEAR
    MOVWF 	T_GEAR		; take a copy
    SUBLW 	D'1'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_2ND		; flag not set so check next gear
	GOTO	GO_NEUTRAL	; flag set so we're in 1st gear
CHK_2ND
	; Not in 1st or neutral so we can activate the Down-Gear routine
	CALL	SOLENOID_DOWN
	DECF	GEAR
	CALL	GEAR_DISP
	NOP
	DELAY_250MS			; debounce SW
	GOTO	SWTEST

UPGEAR
	; 1st check to see if we're in 6th gear yet.
	MOVFW	GEAR
    MOVWF 	T_GEAR		; take a copy
    SUBLW 	D'6'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_6TH		; flag not set so check next gear
	GOTO	SWTEST		; flag set so GEAR=6 already 
CHK_6TH
	; Not in 6th so we have a valid UP shift
	CALL	SOLENOID_UP
	INCF	GEAR
	CALL	GEAR_DISP
	NOP
	DELAY_250MS			; debounce SW
	GOTO	SWTEST

GO_NEUTRAL
	; We want to go into neutral so do the neutral-budge
	CALL	SOLENOID_DOWN
	NOP
	DELAY_125MS 			; Change!!
	CALL	SOLENOID_UP
	CLRF	GEAR			; Set to '-' for Neutral
	CALL	GEAR_DISP
	NOP
	DELAY_250MS				; debounce SW
	GOTO	SWTEST

	GOTO	SWTEST

;*****************************************************************************
; S U B R O U T I N E S
;*****************************************************************************

;*****************************************************************************
SOLENOID_DOWN
	BCF	SOL_DOWN
	DELAY_250MS
	BSF	SOL_DOWN
	RETURN

;*****************************************************************************
SOLENOID_UP
	BCF	SOL_UP
	DELAY_250MS
	BSF	SOL_UP
	RETURN

;*****************************************************************************
WIGGLER
	MOVLW 	b'11111110'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY
	DELAY_250MS
	MOVLW 	b'11111101'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY
	DELAY_250MS
	MOVLW 	b'11111011'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY
	DELAY_250MS
	MOVLW 	b'11110111'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY
	DELAY_250MS
	MOVLW 	b'11101111'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY
	DELAY_250MS
	MOVLW 	b'11011111'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY
	DELAY_250MS
	MOVLW 	b'10111111'
	MOVWF	GEAR_OUT
	CALL	UP_DISPLAY

	RETURN

;*****************************************************************************
; Routine is a case(gear) statement - output is to update the LED7Seg
GEAR_DISP
	MOVFW	GEAR
CHK_0
    MOVWF 	T_GEAR		; take a copy
    SUBLW 	D'0'		; take away 0 and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_1		; flag not set so check next gear
	GOTO	GEAR_0		; flag set so GEAR=0
CHK_1	
	MOVFW	T_GEAR
    SUBLW 	D'1'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_2		; flag not set so check next gear
	GOTO	GEAR_1		; flag set so GEAR=1
CHK_2
	MOVFW	T_GEAR
    SUBLW 	D'2'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_3		; flag not set so check next gear
	GOTO	GEAR_2		; flag set so GEAR=1
CHK_3	
	MOVFW	T_GEAR
    SUBLW 	D'3'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_4		; flag not set so check next gear
	GOTO	GEAR_3		; flag set so GEAR=1
CHK_4	
	MOVFW	T_GEAR
    SUBLW 	D'4'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_5		; flag not set so check next gear
	GOTO	GEAR_4		; flag set so GEAR=1
CHK_5	
	MOVFW	T_GEAR
    SUBLW 	D'5'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	CHK_6		; flag not set so check next gear
	GOTO	GEAR_5		; flag set so GEAR=1
CHK_6
	MOVFW	T_GEAR
    SUBLW 	D'6'		; take away and compare
    BTFSS 	STATUS, Z
    GOTO 	GEAR_ERROR	; Not matched any gears so display E(rror)
	GOTO	GEAR_6		; flag set so GEAR=1
GEAR_ERROR
	MOVLW	DIG_E
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_0
	MOVLW 	DIG_0
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_1
	MOVLW 	DIG_1
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_2
	MOVLW 	DIG_2
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_3
	MOVLW 	DIG_3
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_4
	MOVLW 	DIG_4
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_5
	MOVLW 	DIG_5
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
GEAR_6
	MOVLW 	DIG_6
	MOVWF	GEAR_OUT
	GOTO	WRITEGEAR
WRITEGEAR
	CALL	UP_DISPLAY	
	RETURN

;*****************************************************************************
; Bit twiddling routine to use two wires to output to LED7seg
UP_DISPLAY	; REG: Gear_out has value to be output
	MOVLW	 8
	MOVWF	 i_COUNT	; we count down from 8 to 1, 0=exit

NOLOOP1
	BTFSS	GEAR_OUT, 7
	GOTO 	OUT_0	; BIT IS A 0
	GOTO	OUT_1	; BIT IS A 1
OUT_1
	BSF		SER_DATA
	GOTO	OUT_STRB
OUT_0
	BCF		SER_DATA
OUT_STRB
	CLOCKSTROBE
	RLF		GEAR_OUT, 1	;shift left MSB is O/P First!!!!
	DECFSZ	i_COUNT
   	GOTO	NOLOOP1
	RETURN

;**************************************************************************
; D E L A Y Routines
;**************************************************************************
; It takes 2 instruction cycles to setup delay routine (2)
; Decrementing loop takes 3 instruction cycles per decrement (3 per dec)
; And end branch takes 2 instruction cycles (2)
; So:
; Delay_time	= ((DELAY_value * 3) + 4) * Cycle_time
; Rearranging:
; DELAY_value	= (Delay_time - (4 * Cycle_time)) / (3 * Cycle_time)
;
; i.e. (@ 3.6864MHz crystal)
; Delay_time	= ((14 * 3) + 4) * 1.085uSec
;		= 50uSec
; DELAY_value	= (500uSec - 4) / 3
;		= 152.27
;		= 152
;=============================================================================
; 500usec Delay
DELAY500	MOVLW	D'152'		; +1		1 cycle
		MOVWF	DELAY		; +2		1 cycle
DELAY500_LOOP	DECFSZ	DELAY, F	; step 1	1 cycle
		GOTO	DELAY500_LOOP	; step 2	2 cycles
DELAY500_END	RETURN			; +3		2 cycles
;=============================================================================
; Multiple of 500usec Delay
X_DELAY500	MOVWF	X_DELAY		; +1		1 cycle
X_DELAY500_LOOP	CALL	DELAY500	; step1		wait 500uSec
		DECFSZ	X_DELAY, F	; step2		1 cycle
		GOTO	X_DELAY500_LOOP	; step3		2 cycles
X_DELAY500_END	RETURN			; +2		2 cycles
;=============================================================================

	END				; End of program

